# This file is a part of TkFax.
# fax-init.tcl
#  8 Feb 2004
# This tcl script initiates TkFax.

set fax(debug) 0
if  [ info exists  env(DEBUG) ] { set fax(debug) $env(DEBUG) } 

set fax(docdir) $fax(libdir)/doc

# Read fax source files
source $fax(libdir)/fax-balloon.tcl
source $fax(libdir)/fax-about.tcl
source $fax(libdir)/fax-conf.tcl
source $fax(libdir)/fax-conf-win.tcl
source $fax(libdir)/fax-faxwin.tcl
source $fax(libdir)/fax-lib.tcl
source $fax(libdir)/fax-file.tcl
source $fax(libdir)/fax-mkpg.tcl
source $fax(libdir)/fax-scan.tcl
source $fax(libdir)/fax-pbook.tcl
source $fax(libdir)/fax-queue.tcl
source $fax(libdir)/fax-received.tcl
source $fax(libdir)/fax-logbrowse.tcl
source $fax(libdir)/html_library.tcl
source $fax(libdir)/html_viewer.tcl
source $fax(libdir)/getfont.tcl

# Read scan source files
source $c(libdir)/scan-about.tcl
source $c(libdir)/scan-drivers/$scanner_driver.tcl
source $c(libdir)/scan-conf.tcl
source $c(libdir)/scan-lib.tcl
source $c(libdir)/scan-scan.tcl
source $c(libdir)/scan-out.tcl
source $c(libdir)/scan-scanwin.tcl
source $c(libdir)/scan-file.tcl
source $c(libdir)/scan-win.tcl

#set fax(incomingdir) ""
set fax(userconfigfile) $fax(userdir)/config.tcl

proc autolocale {} {
  global env fax c m
  #  6 Oct 2002 localemsg ,look up LS_ALL etc.
  # 23 Jan 2002 auto scan the lolale
  set fax(locale) $fax(usrlocale)
  set envlocale {}
  if { $fax(locale) == {}  } {
	foreach i  {LANGUAGE LC_ALL LANG } {
		if { [ info exists  env($i) ] } { set envlocale $env($i) } 
	}
      if  { [ string length $envlocale ] >= 2 } {
          set fax(locale) [ string range $envlocale 0 1 ]
      }
  }
  # set locale and read messages
  foreach i { scan-msg.tcl fax-msg.tcl } {
     source $fax(docdir)/C/$i
  }
  if { $fax(locale)=="" } { set fax(locale) "C" }
  if { $fax(locale)!="C"} {
	set localemsgdir $fax(docdir)/$fax(locale)
	foreach i { scan-msg.tcl fax-msg.tcl } {
	    if [file exists $localemsgdir/$i] { source $localemsgdir/$i }
	}
  }
  return $fax(locale)
}

proc fax_userdir_check {} {
#
# Check if $fax(userdir) exists. If not, creates it properly.
#
# 6 Oct 2002  use tcl commands but ecape 'exec'
    global fax m
    if [file exist $fax(userdir)] {
        if [file isdirectory $fax(userdir)] {
            foreach dir {texdir pbookdir tmpdir} {
                if [file exist $fax($dir)] {
                    if [file isdirectory $fax($dir)] {
                    } else {
                        puts "$fax($dir) exists, but not a directory."
                    }
                } else {
                    file mkdir $fax($dir)
                }
            }
        } else {
            puts "$fax(userdir) exists, but not a directory."
        }
    } else {
        foreach dir {userdir texdir pbookdir tmpdir} {
            file mkdir $fax($dir)
        }
    }
    set f $fax(pbookdir)/group.pbd
    if { ! [file exist $f] } {
	set fid [open $f w]
	puts $fid "Misc\tMiscellaneous fax numbers"
	close $fid
    }
    set f $fax(pbookdir)/Misc.pbd
    if { ! [file exist $f] } {
	set fid [open $f w]
	close $fid
    }
}

proc autoconfig {} {
  global env fax c m paperList openingmsg
  set e ""

  # file names
  set fax(flagfile)  $fax(tmpdir)/flag
  set fax(pbookgroupfile)  $fax(pbookdir)/group.pbd

  # Check if userdir exists. If not, create it.
  fax_userdir_check

  # initiate tkfax logfile
  set username [lindex [file split $fax(userdir)] 2]
  set tkfaxlog "tkfax_${username}.log"
  if [file writable $fax(logdir)] {
      set fax(tkfaxlogfile)  $fax(logdir)/$tkfaxlog
      set fax(startlogdir) $fax(logdir)
  } else {
      append e "$fax(logdir) "
      set fax(tkfaxlogfile)  $fax(userdir)/$tkfaxlog
      set fax(startlogdir) $fax(userdir)
  }
  set fid [open $fax(tkfaxlogfile) w]
  puts $fid "$fax(progname) version $fax(ex_version), $fax(ex_date)"
  puts $fid [ clock format [clock seconds] ]
  if { $e != "" } {
     set code [catch "close $fid"]
     catch [exec sh -c "ls -dl $fax(logdir)  >> $fax(tkfaxlogfile) 2>&1" ]
     set fid [open $fax(tkfaxlogfile) a]
     puts $fid "$e not available."
  }
  set code [catch "close $fid"]

  # Some xternal programs
  set commands {efax sendpage faxspool}
  set agents {efax hylafax sendfax}
  set fax(faxcommand) ""
  set j 0
  foreach i $commands {
    if { $fax(faxagent) == "" } {
	if [excutable $i] {
		set fax(faxcommand) $i
		set fax(faxagent) [lindex $agents $j]
	 }
    }
    incr j
  }

  if { ! [ info exists fax(viewers) ] } { set fax(viewers) {} }
  if { $fax(viewers) == {} } {
      foreach i {{fax view} viewfax kfax ghfaxviewer xv {display -geometry 400x600} {xloadimage -zoom 35} } {
           if [excutable $i] { lappend fax(viewers) $i } 
      }
  }
  if { ! [ info exists fax(viewer) ] } { set fax(viewer) {} }
  if { $fax(viewer) == {} } {set fax(viewer) [lindex $fax(viewers) 0] }

  if { ! [ info exists fax(xterms) ] } { set fax(xterms) {} }
  if { $fax(xterms) == {} } {
      lappend fax(xterms) none 
      foreach i "kterm rxvt xterm" {
           if [excutable $i] { lappend fax(xterms) $i } 
      }
  }
  if { ! [ info exists fax(xterm) ] } { set fax(xterm) {} }
  if { $fax(xterm) == {} } {set fax(xterm) [lindex $fax(xterms) 0] }

  if { ! [ info exists fax(latexs) ] } { set fax(latexs) {} }
  if { $fax(latexs) == {} } {
      foreach i "platex latex" {
           if [excutable $i] { lappend fax(latexs) $i } 
      }
     lappend fax(latexs) {none}
  }
  if { ! [ info exists fax(latex) ] } { set fax(latex) {} }
  if { $fax(latex) == {} } {set fax(latex) [lindex $fax(latexs) 0] }
  if  { $fax(latex) == "none" } {
      set fax(uselatex) 0
  } else {
      set fax(uselatex) 1 
  }
  if { $fax(latex) == "platex" && [ excutable "pdvips" ] } {
      set fax(dvips) {pdvips}
  }

  if { ! [ info exists fax(mpages) ] } { set fax(mpages) {} }
  if { $fax(mpages) == {} } {
      foreach i {{a2ps --columns=1 --borders no --margin=30 -R -B} \
			{a2ps --encoding=euc-jp --columns=1 --borders no --margin=30 -R -B} \
			{a2ps-j --columns=1 --borders no --margin=30 -R -B} \
			{mpage -1 -o -m30l} } {
           if [excutable $i] { lappend fax(mpages) $i } 
      }
  }
  if { ! [ info exists fax(mpage) ] } { set fax(mpage) {} }
  if { $fax(mpage) == {} } {set fax(mpage) [lindex $fax(mpages) 0] }

  if { ! [ info exists fax(browsers) ] } { set fax(browsers) {} }
  if { $fax(browsers) == {} } {
      set fax(browsers) "tk"
      if { [ info exists  env(BROWSER) ] } {
          if { $env(BROWSER) != {} } { lappend fax(browsers) $env(BROWSER) }
      }
      foreach i "galeon gnome-help-browser {gnome-moz-remote --raise --newwin}  konqueror lynx mozilla netscape nautilus  opera w3m w3m-term " {
              if [excutable $i] { lappend fax(browsers) $i }
      }
  }
  if { ! [ info exists fax(browser) ] } { set fax(browser) {} }
  if { $fax(browser) == {} } { set fax(browser) "tk" }

  foreach i "lpr lp" {
    if { $fax(printcmd) == "" } { if [excutable $i] { set fax(printcmd) $i } }
  }

  # depends on the fax agent
  #  7 Feb 2003
      set fax(sendfax)	{}
      set fax(receivefax)   {}
      set fax(faxspool)	{}
      set fax(faxqueue)	{}
      set fax(faxrm) {}
      set fax(g32pbm)	{}
      set fax(pbm2g3)	{}

 switch -- $fax(faxagent) {
    sendfax {
      set fax(sendfax)	{/usr/sbin/sendfax -v}
      set fax(receivefax)   {}
      set fax(faxspool)	{faxspool}
      set fax(faxqueue)	{faxq}
      set fax(faxrm) {faxrm}
      set fax(g32pbm)	{g32pbm}
      set fax(pbm2g3)	{pbm2g3}
  #  if  { $fax(viewer) == {} } { set fax(viewer) "viewfax" }
      if  { $fax(spooldir) == "" } { set fax(spooldir) "/var/spool/fax" } 
      if  { $fax(incomingdir) == "" } { set fax(incomingdir) "$fax(spooldir)/incoming" }
      if  { $fax(outgoingdir) == "" } { set fax(outgoingdir) "$fax(spooldir)/outgoing" }
      if  { $fax(logdir) == "" } { set fax(logdir) "/var/log/fax" }
    }
    efax  {
      set fax(sendfax)	{fax send -v}
      set fax(receivefax) {fax receive}
      set fax(faxspool)	{lpr -P fax -J}
      set fax(faxqueue)	{lpq -P fax}
      set fax(faxrm) {lprm -P fax}
      set fax(g32pbm)	{efix -i tiffg3 -o pbm}
      set fax(pbm2g3)	{efix -i pbm -o tiffg3}
  #  if  { $fax(viewer) == {} } { set fax(viewer) "fax view" }
  #  if  { $fax(printertype) == {} } { set fax(printertype) "efax" }
      if  { $fax(spooldir) == "" } { set fax(spooldir) "/var/spool/fax" } 
      if  { $fax(incomingdir) == "" } { set fax(incomingdir) "$fax(spooldir)/incoming" }
      if  { $fax(outgoingdir) == "" } { set fax(outgoingdir) "$fax(spooldir)/outgoing" }
      if  { $fax(logdir) == "" } { set fax(logdir) "/var/log/fax" }
    }
    hylafax  {
      set fax(sendfax)	{sendfax -n -d}
      set fax(receivefax) {}
      set fax(faxspool)	{sendfax -n -d}
      set fax(faxqueue)	{faxstat}
      set fax(faxrm) {faxrm}
      set fax(g32pbm)	{false}
      set fax(pbm2g3)	{false}
  #    set fax(ps2fax) {/var/spool/fax/bin/ps2fax}
	set fax(spooling) 1
      if  { $fax(spooldir) == "" } {
	foreach i "hylafx fax" {
		set d "/var/spool/$i"
		if [ file isdirectory $d ] { set fax(spooldir) $d } 
	}
      }
      if  { $fax(incomingdir) == "" } { set fax(incomingdir) "$fax(spooldir)/recvq" }
      if  { $fax(outgoingdir) == "" } { set fax(outgoingdir) "$fax(spooldir)/outgoing" }
      if  { $fax(logdir) == "" } { set fax(logdir) "$fax(spooldir)/log" }
    }
   }

  # essencial programs
  set fax(pgmtopbm)	{pgmtopbm}
  foreach i "sendfax viewer convert mpage xterm printcmd ghostscript pgmtopbm" {
      if { $fax($i) == "" || ! [excutable $fax($i)] } {
	   set missing ""
	   switch -- $i {
		sendfax { set missing "faxagent" }
		convert { set missing "ImageMagic" }
		xterm   { if { "$fax(xterm)" != "none" } {
                                set missing  [lindex [split $fax($i) " "] 0]
			  }
			}
	        default { set missing [lindex [split $fax($i) " "] 0] }
	   }	   
           if { "$missing" != "" } { append e "$missing " }
      }
  }
  #  5 Mar 2002
  set c(pbmtofax) $fax(pbm2g3)

  autolocale

  # Using japanese require some special programs
  # 7 Sep 2001 use procedure 'excutable'
  # 3 Sep 2001
  if { $fax(locale) == "ja" } {
      if { $fax(uselatex) && \
           [ excutable "platex" ] } {
          set fax(latex)	{platex}
      }
      #if [ excutable "kterm"] {
      #      set fax(xterm)	{kterm}
      #}
      #
      # 3 Sep 2001
      set p 0
      foreach lang $fax(custom) {
        if { [lindex $lang 0 ] == "Japan" } {
            set fax(custom) [lreplace $fax(custom) $p $p { {ܸ} ja }]
            set fax(customstyle) $p
        }
        set p [expr $p+1]
      }
      # 4 Sep 2001 for Tcl/Tk8.3
      catch { tk useinputmethods 1 }
  }

  # calcurate dimensions
  # 4 Sep 2001 Asume printerYreso is = printerXreso 
  # 1 Sep 2001
  if { $fax(printerYreso) == 0 } { set fax(printerYreso) $fax(printerXreso) }
  foreach j {fax printer} {
    set flag {}
    while { $flag == {} } {
       foreach i $paperList {
       set P  [lindex $i 0]  
         if { $P == $fax(${j}Paper) } {
            set flag P
            set size(X) [lindex $i 1]
            set size(Y) [lindex $i 2]
         }
      }
      if { $flag == {} } { set flag {a4}; set fax(${j}Paper $flag }
    }
    foreach i {X Y} {
      set fax($j${i}inch) $size($i)
      set fax($j${i}dots) [ expr 8*int(($size($i)-2*$fax($j${i}margin))*$fax($j${i}reso)/8) ]
    }
  }

  #
  set e [string trim $e]
  if { $e != "" } { set e "$e is_not_available" }
  set fax(missing) $e
  return $e
}

proc msgtranslate { sourcemsg } {
   global fax m
   set msg ""
   foreach i $sourcemsg {
      if [ info exists m($i) ] {
      set msg "$msg $m($i)" } else { set msg "$msg $i" }
   }
   return $msg
}

# save configurations
autoconfig
foreach i [array names fax *] {
    if { "$i" == "debug" }      { continue } 
    set faxsave($i) $fax($i)
}
foreach i [array names cover *] {
    set coversave($i) $cover($i)
}

# Read user configuration file in if exists.
# 7 Sep 2001 check config_version
# **** Variables do not change when this function reads user's config file!
# **** Next script works OK. (by R.Kawagishi)
set openingmsg ""
if [file exist $fax(userconfigfile)] {
    set fax(config_version) 0
    source $fax(userconfigfile)
    if { [string compare  $fax(config_version) "0.99b"] < 0 } {
         set openingmsg "old_config edit_config"
    }
} else {
#    file copy $fax(libdir)/fax-conf.tcl $fax(userconfigfile)
#    set openingmsg "new_config edit_config"
    set openingmsg "edit_config"
}


# load message
autolocale

# Lock tmpdir
#  8 Feb 2004 Masaki, use PID
set lockfile $fax(tmpdir)/lock
set fax(lockfile) $lockfile
if [file exists $lockfile] {
    set fid [open $lockfile r]
    gets $fid pid
    close $fid
    set notrun 1
    catch {set notrun [exec sh -c "ps -p $pid > /dev/null; echo $?"]}
    if {! $notrun} {
        if  [tk_dialog .dialog "TkFax lock file found" \
            $m(multi_run)  \
             {warning} 1 $m(confirm) $m(cancel)] { exit }
    }
}
# get my PID
set pid [exec sh -c "ps x | grep -v grep |grep 'wish .*tkfax'| tail -n 1|cut -d' ' -f1"]
set fid [open $lockfile w]
puts $fid $pid
close $fid


# Check the configuration
set e [autoconfig]
if { $e != "" } { set openingmsg $e }

# Build up phone book database
fax_pbook_read_faxnumberlist

# Initialize some variables
#
set fax(c) 0
fax_number_clear
set fax(queuelist) 0
set pbook_modified_flag 0
set fax(removed_grouplist) {}

# Launch the main window
# 20 Sep 2001 --debug
set arg1 [lindex $argv 0]
switch -- $arg1 {
    -r fax_received_window
    -q fax_queue_window
    --debug -
    -debug
       { set fax(debug) 1
         fax_main_window {}
        }
    -s -
    default {fax_main_window {}}
}

# opening message
fax_message [msgtranslate $openingmsg]

# End
